@@ -332,10 +332,10 @@
 "*Dialog.borderWidth:		1",
 "*Dialog.shadowThickness:	2",
 "*passwd.heading.label:		XScreenSaver %s",
-"*passwd.body.label:		Please enter your password.",
+"*passwd.body.label:		This screen is locked.",
+"*passwd.unlock.label:		OK",
 "*passwd.login.label:		New Login",
 "*passwd.user.label:		Username:",
-"*passwd.passwd.label:		Password:",
 "*passwd.thermometer.width:	8",
 "*passwd.asterisks:              True",
 "*passwd.uname:                  True",
