@@ -105,10 +105,6 @@
 #include "screenhackI.h"
 #include "version.h"
 #include "vroot.h"
-#include "change_locale.h"
-#if HAVE_SETLOCALE
-#include <locale.h>
-#endif
 
 #ifndef _XSCREENSAVER_VROOT_H_
 # error Error!  You have an old version of vroot.h!  Check -I args.
@@ -162,27 +158,6 @@ static XrmOptionDescRec *merged_options;
 static int merged_options_size;
 static char **merged_defaults;
 
-static void 
-reset_locale (void)
-{
-#ifdef HAVE_SETLOCALE
-  const char *current_locale = setlocale(LC_ALL, "");
-  const char *cmp_locale;
-  int j = 0;
-
-  while ((cmp_locale = change_locale[j]))
-    {
-      if (!strncmp(current_locale, cmp_locale, strlen(cmp_locale)))
-	{
-	  setlocale(LC_ALL, "C");
-	  break;
-	}
-      j++;
-    }
-#endif /* HAVE_SETLOCALE */
-}
-
-
 static void
 merge_options (void)
 {
@@ -709,7 +684,7 @@ init_window (Display *dpy, Widget toplevel, const char *title)
   window = XtWindow (toplevel);
   XGetWindowAttributes (dpy, window, &xgwa);
   XSelectInput (dpy, window,
-                (xgwa.your_event_mask | KeyPressMask |
+                (xgwa.your_event_mask | KeyPressMask | KeyReleaseMask |
                  ButtonPressMask | ButtonReleaseMask));
   XChangeProperty (dpy, window, XA_WM_PROTOCOLS, XA_ATOM, 32,
                    PropModeReplace,
@@ -738,7 +713,6 @@ main (int argc, char **argv)
   char version[255];
 
   fix_fds();
-  reset_locale();
 
   progname = argv[0];   /* reset later */
   progclass = ft->progclass;
